<?php

namespace Materials\Material\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Materials\Material\Http\Requests\MaterialAdminWebRequest;
use Materials\Material\Interfaces\MaterialRepositoryInterface;
use Materials\Material\Models\Material;

/**
 * Admin web controller class.
 */
class MaterialAdminController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'admin.web';

    /**
     * Initialize material controller.
     *
     * @param type MaterialRepositoryInterface $material
     *
     * @return type
     */
    public function __construct(MaterialRepositoryInterface $material)
    {
        $this->middleware('web');
        $this->middleware('auth:admin.web');
        $this->setupTheme(config('theme.themes.admin.theme'), config('theme.themes.admin.layout'));

        $this->repository = $material;
        parent::__construct();
    }

    /**
     * Display a list of material.
     *
     * @return Response
     */
    public function index(MaterialAdminWebRequest $request)
    {
        $pageLimit = $request->input('pageLimit');
        if ($request->wantsJson()) {
            $materials  = $this->repository
            ->pushCriteria(new \Materials\Material\Repositories\Criteria\MaterialAdminCriteria())
                ->setPresenter('\\Materials\\Material\\Repositories\\Presenter\\MaterialListPresenter')
                ->scopeQuery(function ($query) {
                    return $query->orderBy('id', 'DESC');
                })->paginate($pageLimit);

            $materials['recordsTotal']    = $materials['meta']['pagination']['total'];
            $materials['recordsFiltered'] = $materials['meta']['pagination']['total'];
            $materials['request']         = $request->all();
            return response()->json($materials, 200);

        }

        $this   ->theme->prependTitle(trans('material::material.names').' :: ');
        return $this->theme->of('material::admin.material.index')->render();
    }

    /**
     * Display material.
     *
     * @param Request $request
     * @param Model   $material
     *
     * @return Response
     */
    public function show(MaterialAdminWebRequest $request, Material $material)
    {
        if (!$material->exists) {
            return response()->view('material::admin.material.new', compact('material'));
        }

        Form::populate($material);
        return response()->view('material::admin.material.show', compact('material'));
    }

    /**
     * Show the form for creating a new material.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(MaterialAdminWebRequest $request)
    {

        $material = $this->repository->newInstance([]);

        Form::populate($material);

        return response()->view('material::admin.material.create', compact('material'));

    }

    /**
     * Create new material.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(MaterialAdminWebRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $material          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('material::material.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/material/material/'.$material->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show material for editing.
     *
     * @param Request $request
     * @param Model   $material
     *
     * @return Response
     */
    public function edit(MaterialAdminWebRequest $request, Material $material)
    {
        Form::populate($material);
        return  response()->view('material::admin.material.edit', compact('material'));
    }

    /**
     * Update the material.
     *
     * @param Request $request
     * @param Model   $material
     *
     * @return Response
     */
    public function update(MaterialAdminWebRequest $request, Material $material)
    {
        try {

            $attributes = $request->all();

            $material->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('material::material.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/material/material/'.$material->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/material/material/'.$material->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the material.
     *
     * @param Model   $material
     *
     * @return Response
     */
    public function destroy(MaterialAdminWebRequest $request, Material $material)
    {

        try {

            $t = $material->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('material::material.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/material/material/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/material/material/'.$material->getRouteKey()),
            ], 400);
        }
    }
}
